/*
** Module   : MenuContainerWidget.java
** Abstract : The base class for MenuWidget and SubMenuWidget.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 VIG 20150115          Created initial version.
*/
package com.goldencode.p2j.ui;

import java.util.Iterator;
import java.util.Map;

/**
 * This class represents sub-menu widget.
 */
abstract class MenuContainerWidget<T extends PaneConfig>
extends PaneEntity<T>
{
   /** Maps widget names to widget instances (from the menu definition). */
   private Map<String, GenericWidget<?>> n2w = null;

   /** List of widgets in ID order. */
   private GenericWidget<?>[] widgets = null;
   
   /** menu definition */
   private MenuDescription menuDef = null;
   
   /** Shared menu flag. */
   private boolean shared = false;

   /** batch modifications flag */
   private boolean batch = true;
   
   /**
    * Default constructor.  
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    * @param    menuConfig
    *           The menu config instance.
    */
   public MenuContainerWidget(boolean dynamic, T menuConfig)
   {
      super(dynamic, menuConfig);
   }

   /**
    * Check if menu is shared.
    * 
    * @return  <code>true</code> if menu is shared.
    */
   private boolean isShared()
   {
      return shared;
   }
   
   /**
    * Set shared flag for menu.
    * 
    * @param   shared
    *          <code>true</code> if menu is shared.
    */
   void setShared(boolean shared)
   {
      this.shared = shared;
   }
   
   /**
    * Sets batching mode for frame modification.
    *
    * @param  on
    *         <code>true</code> sets batching mode
    */
   public void batch(boolean on)
   {
      batch(on, false);
   }

   /**
    * Queries batching mode for frame modification.
    *
    * @return 
    *         <code>true</code> if batching mode is in effect
    */
   public boolean isBatch()
   {
      return batch;
   }

   /**
    * Sets batching mode for frame modification and optionally pushes the
    * screen definition to the client. 
    *
    * @param  on
    *         <code>true</code> sets batching mode
    * @param  force
    *         <code>true</code> to ensure that the screen definition will be
    *         synchronously pushed to the client. <code>false</code> to
    *         defer the push. This is only honored when <code>on</code> is
    *         <code>false</code>.
    */
   public void batch(boolean on, boolean force)
   {
      batch = on;
      
      if (!batch)
         pushMenuDescription(force);
   }
   
   /**
    * Core initialization processing. 
    *
    * @param    defClass
    *           The class of the definition implementation class.
    * @param    defInstance
    *           The instance of the definition implementation class.
    * @param    dynamic
    *           <code>true</code> If this menu is dynamic, <code>false</code> otherwise.
    */
   protected void coreInitialize(Class<?>         defClass,
                               Object             defInstance, 
                               boolean            dynamic)
   {
      WidgetListBase<?> wl = (WidgetListBase<?>) defInstance;
 
      // cache the widget map for this menu
      n2w = wl.getWidgets();
      
      // init the widget array
      widgets = new GenericWidget[n2w.size()];
      
      // dynamic menu initialization should stop here
      if (dynamic)
      {
         return;
      }

      // widget array counter      
      int idx = -1;

      // add all the widgets in the order of their definition
      Iterator<String> iter = n2w.keySet().iterator();
      
      while (iter.hasNext())
      {
         // calculate the array index
         idx++; 
         
         int wid = WidgetId.nextID().getId();
         
         // get the widget
         widgets[idx] = n2w.get(iter.next());
         
         // initialize key state in the widget
         widgets[idx].setId(wid);

         // save new widget in session registry map
         LogicalTerminal.registerWidget(widgets[idx], wid);
      }
   }
   
   /**
    * Conditionally pushes the screen definition instance down to the client. 
    * If modifications are currently being batched, no action is taken.
    * Otherwise, a new screen definition is built and sent to the client.
    * <p>
    * When the screen definition is built, a method of 
    * <code>LogicalTerminal</code> is called to send it down to the client.
    * 
    * Overrides {@link GenericWidget#pushScreenDefinition()} behavior.
    */
   protected void pushScreenDefinition()
   {
      pushMenuDescription(true);
   }
   
   /**
    * Conditionally pushes the screen definition instance down to the client. 
    * If modifications are currently being batched, no action is taken.
    * Otherwise, a new screen definition is built and sent to the client.
    * <p>
    * When the screen definition is built, a method of 
    * <code>LogicalTerminal</code> is called to send it down to the client.
    * 
    * @param  force
    *         <code>true</code> to ensure that the screen definition will be
    *         synchronously pushed to the client. <code>false</code> to
    *         defer the push.
    */
   void pushMenuDescription(boolean force)
   {
      // batching is in progress, do not push individual changes
      if (batch)
         return;
      
      CommonWidget parent = getParent();
      if (parent != null && parent instanceof MenuContainerWidget<?>)
      {
         ((MenuContainerWidget<?>)parent).pushScreenDefinition();
      }

      // time to push the changes down
      if (menuDef == null)
      {
         // create a new menuDef
         int menuId = config.id.getId();
         menuDef = new MenuDescription(menuId);
         menuDef.addConfig((MenuElementConfig) config, menuId);

         // add all the widgets in the order of their definition
         
         MenuElementConfig cfg = null;
         for (int i = 0; i < widgets.length; i++)
         {
            GenericWidget<?> gw = widgets[i];
            
            cfg = (MenuElementConfig) gw.getConfig();
            menuDef.addConfig(cfg);
         }
         
         if (isShared() && isVisible().booleanValue())
            menuDef.setSuppressRedraw(true);
      }
      
      // push down the screen definition
      if (force)
      {
         // synchronous push
         MenuDescription[] sd = new MenuDescription[] {menuDef};
         LogicalTerminal.pushMenuDescription(sd);
      }
      else
      {
         // deferred push
         LogicalTerminal.deferPush(menuDef);
      }
   }
   
   /**
    * Finish the config processing for this frame.
    * <p>
    * This includes registering the active configuration for each new widget, and synchronizing
    * the widget configuration with the actual widget state, by calling the 
    * {@link WidgetConfig#syncWithWidget()} API.
    */
   void finishConfigProcessing()
   {
      ConfigManager mgr = ConfigManager.getInstance();
      
      for (GenericWidget<?> w : widgets)
      {
         WidgetConfig cfg = w.config;

         WidgetConfig active = mgr.getActiveConfig(cfg.id);
         if (active == null)
         {
            mgr.addWidgetConfig(cfg);
         }
         else
         {
            assert(cfg.getClass().equals(active.getClass()));
         }
   
         // we need to sync with the widget here, to ensure the processing which was previously
         // done in custom setters is not lost
         
         cfg.syncWithWidget();
      }
   }

}
