/*
** Module   : WindowConfig.java
** Abstract : window widget attribute container.
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 SIY 20050831   @22524 Created initial version. Part of refactoring
**                           (see @@22394-22409). 
** 002 NVS 20051128   @23543 Added fields and methods to serve window
**                           attributes max-height-chars, max-width-chars.
** 003 SIY 20060201   @24213 Refactoring and cleanups.      
** 004 NVS 20060608   @27012 Assigned a permanent ID to the session window
** 005 GES 20060802   @28421 Made externalizable for better performance.
**                           Many code standard cleanups.
** 006 GES 20061120   @31334 Removed dead code.
** 007 SIY 20100619          Switching to own implementation of character UI.
** 008 SIY 20100804          Changes in imports, some refactorings.
** 009 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 010 MAG 20140512          Implemented new attributes and support for CREATE WINDOW.
** 011 CA  20140711          Enhanced font support, for GUI clients.
** 012 CA  20140805          Do not set the window ID upfront, to allow multi-window support.  
**                           Added GUI-only hack to allow the window title to be pushed from the 
**                           server-side to client-side, without overriding other config.
** 013 MAG 20140802          Add environment name associated with window. 
** 014 MAG 20140909          Fixed NPE on serialization of envName. 
** 015 EVL 20140812          In new widget ID allocation approach ID for default window is taking
**                           from WidgetId class static constant.
** 016 HC  20140911          GUI coordinate support - data types refactoring.
** 017 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 018 VIG 20141024          Added RESOLVE_WINDOW constant.
** 019 CA  20141119          Refactored drawing code to enable multiple OS windows, for GUI
**                           support.  Misc changes/fixes related to drawing on multiple windows.
** 020 CA  20141123          No special processing is required when setting the window's COLUMN,
**                           ROW, WIDTH-CHARS or HEIGHT-CHARS attributes.  Also, the WIDTH-/HEIGHT-
**                           CHARS attributes need to reflect the workspace's dimension.
** 021 EVL 20150108          Differentiate small icon from large one flag.  Rework for icon/image
**                           implementation.
** 022 VIG 20150120          Added menubarId.
*/

package com.goldencode.p2j.ui;

import java.io.*;

import com.goldencode.p2j.ui.client.widget.*;

/**
 * Window widget configuration container.
 */
public class WindowConfig
extends PaneConfig
{
   /** 
    * Window id which used in calls of client and means, that target window in the client-side
    * must be resolved automatically.
    */
   public static final int RESOLVE_WINDOW = -1;

   /** Normal window icon size. */   
   public static final int ICON_SIZE = 32;
   
   /** Small sized window icon size. */   
   public static final int SMALL_ICON_SIZE = 16;
   
   /** Default application icon directory section. */
   public static final String APP_ICON_SECTION = "window-icon";
   
   /** Default application icon. */
   public static final String DEFAULT_ICON = "gclogo.ico";
   
   /** The Progress FULL-HEIGHT-CHAR attribute. */
   public double fullHeightChar = 0;
   
   /** The Progress SCREEN-LINES attribute. */
   public double screenLines = 0;
   
   /** The Progress KEEP-FRAME-Z-ORDER attribute. */
   public boolean keepFrameZOrder = false;
   
   /** The Progress NUM-SELECTED-WIDGET attribute. */
   public int numSelectedWidget = 0;

   /** The Progress SESSION:DATA-ENTRY-RETURN attribute. */
   public boolean sessDataEntryReturn = true;

   /** The Progress FULL-WIDTH-CHAR attribute. */
   public double fullWidthChar = 0;
   
   /** MAX-HEIGHT-CHARS attribute */
   public double maxHeightChars = 0;

   /** MAX-WIDTH-CHARS attribute */
   public double maxWidthChars = 0;

   /** MAX-HEIGHT-PIXELS attribute */
   public int maxHeightPixels = 0;

   /** MAX-WIDTH-PIXELS attribute */
   public int maxWidthPixels = 0;
   
   /** HELP attribute. */
   public String help = null;

   /** The Progress RESIZE attribute. */
   public boolean resize = true;
   
   /** The Progress SCROLL-BARS attribute. */
   public boolean scrollBars = false;
   
   /** The Progress STATUS-AREA attribute. */
   public boolean statusArea = true;

   /** The Progress MESAGE-AREA attribute. */
   public boolean messageArea = true;
   
   /** The Progress ICON attribute used by LOAD-ICON(). */
   public String iconFileName;
   
   /** Flag indicating the icon was loaded by LOAD-SMALL-ICON(). */
   public boolean useSmallIcon = false;

   /** The font for the window's message area. */
   public int messageAreaFont;
   
   /** The font for the window's status area. */
   public int statusAreaFont;
   
   /** Environment name */
   public String envName;
   
   /** ID of menubar */
   public int menubarId = 0;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public WindowConfig()
   {
      this.visible = false;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected WindowConfig(int id)
   {
      super(id);

      this.visible = false;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected WindowConfig(WidgetId id)
   {
      super(id);
      
      this.visible = false;
   }

   /**
    * Sets value of the ROW attribute.  This is a 0-based row position
    * and NOT the 1-based position used in Progress.
    * 
    * @param    row
    *           The new row value for the upper left component corner relative
    *           to it's parent.
    */
   @Override
   public void setRow(double row)
   {
      row = Coordinate.scale(row);
      
      this.row = row; // direct access
   }
   
   /**
    * Sets column coordinate of the upper left corner of the component
    * relative to the parent container.  This is a 0-based column position
    * and NOT the 1-based position used in Progress.
    * 
    * @param    column
    *           The new column value of upper left component corner.
    */
   @Override
   public void setColumn(double column)
   {
      column = Coordinate.scale(column);
      
      this.column = column; // direct access
   }
   
   /**
    * Set the WIDTH-CHARS Progress attribute.
    * 
    * @param    widthChars
    *           The integer value for the new WIDTH-CHARS value.
    */
   @Override
   public void setWidthChars(double widthChars)
   {
      widthChars = Coordinate.scale(widthChars);
      
      this.widthChars = widthChars;
   }
   
   /**
    * Sets HEIGHT-CHARS Progress attribute.
    * 
    * @param    heightChars
    *           The integer value for the new HEIGHT-CHARS value.
    */
   @Override
   public void setHeightChars(double heightChars)
   {
      heightChars = Coordinate.scale(heightChars);
      
      this.heightChars = heightChars;
   }
   
   /**
    * Synchronize the configuration with the widget state.
    */
   @Override
   public void syncWithWidget()
   {
      super.syncWithWidget();

      if (title == null)
         title = "";
      
      if (id == null || !onClientSide())
      {
         return;
      }

      Widget<?> instance = tc.getWidget(id);
      
      if (instance == null)
      {
         return;
      }

      if (instance.isVisible())
      {
         instance.repaint();
      }
   }
   
   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      WindowConfig cfg = (WindowConfig) config;
      
      fullHeightChar = cfg.fullHeightChar;
      screenLines = cfg.screenLines;
      keepFrameZOrder = cfg.keepFrameZOrder;
      numSelectedWidget = cfg.numSelectedWidget;
      sessDataEntryReturn = cfg.sessDataEntryReturn;
      fullWidthChar = cfg.fullWidthChar;
      maxHeightChars = cfg.maxHeightChars;
      maxWidthChars = cfg.maxWidthChars;
      maxHeightPixels = cfg.maxHeightPixels;
      maxWidthPixels = cfg.maxWidthPixels;
      help = cfg.help;
      resize = cfg.resize;
      scrollBars = cfg.scrollBars;
      statusArea = cfg.statusArea;
      messageArea = cfg.messageArea;
      iconFileName = cfg.iconFileName;
      useSmallIcon = cfg.useSmallIcon;
      messageAreaFont = cfg.messageAreaFont;
      statusAreaFont = cfg.statusAreaFont;
      envName = cfg.envName;
      menubarId = cfg.menubarId;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      fullHeightChar      = in.readDouble();
      fullWidthChar       = in.readDouble();
      screenLines         = in.readDouble();
      numSelectedWidget   = in.readInt();
      keepFrameZOrder     = in.readBoolean();
      sessDataEntryReturn = in.readBoolean();
      maxHeightChars      = in.readDouble();
      maxWidthChars       = in.readDouble();
      maxHeightPixels     = in.readInt();
      maxWidthPixels      = in.readInt();
      resize              = in.readBoolean();
      scrollBars          = in.readBoolean();
      statusArea          = in.readBoolean();
      messageArea         = in.readBoolean();
      iconFileName        = (String) in.readObject();
      useSmallIcon        = in.readBoolean();
      help                = (String) in.readObject();
      messageAreaFont     = in.readInt();
      statusAreaFont      = in.readInt();
      envName             = (String) in.readObject();
      menubarId           = in.readInt();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeDouble(fullHeightChar);
      out.writeDouble(fullWidthChar);
      out.writeDouble(screenLines);
      out.writeInt(numSelectedWidget);
      out.writeBoolean(keepFrameZOrder);
      out.writeBoolean(sessDataEntryReturn);
      out.writeDouble(maxHeightChars);
      out.writeDouble(maxWidthChars);
      out.writeInt(maxHeightPixels);
      out.writeInt(maxWidthPixels);
      out.writeBoolean(resize);
      out.writeBoolean(scrollBars);
      out.writeBoolean(statusArea);
      out.writeBoolean(messageArea);
      out.writeObject(iconFileName);
      out.writeBoolean(useSmallIcon);
      out.writeObject(help);
      out.writeInt(messageAreaFont);
      out.writeInt(statusAreaFont);
      out.writeObject(envName);
      out.writeInt(menubarId);
   }
}
