/*
** Module   : UiUtils.java
** Abstract :
**
** Copyright (c) 2010-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SIY 20100821 Created initial version
** 002 SIY 20100831 Common static method containsFrames moved here.
** 003 SIY 20100915 More common static methods moved here.
** 004 SIY 20101015 Static methods from Screen moved here.
** 005 SIY 20110112 Changes related to new color handling scheme.
** 006 SIY 20111101 Moved from CHUI package. Minor cleanups/refactorings in preparation for GUI. 
** 007 SIY 20111202 Temporary changes caused by splitup of widgets into UI-specific and
**                  UI-independent parts. Will be replaced with widget factory.
** 008 SIY 20111221 Switched to widget factory.
** 009 SVL 20120214 Added locateAlertBox.
** 010 MAG 20140410 Add RECTANGLE widget creation and simple round up method.
** 011 EVL 20140610 Added TOGGLE-BOX widget support.
** 012 MAG 20140627 Add Window class to reconstructComponent method. Changed color attribute
**                  method names to match naming standards.
** 013 CA  20140805 Changes related to eliminating the warnings from adding generics to the 
**                  abstract widgets. 
** 014 EVL 20140812 In new widget ID allocation approach ID for default window is taking from
**                  WidgetId class static constant.
** 015 HC  20140911 GUI coordinate support - data types refactoring.
** 016 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 017 VIG 20141025 Added multi-window support for frame methods.
** 018 CA  20141123 The current focus is always obtained from the top window.
** 019 EVL 20141126 Content pane GUI fix.
** 020 EVL 20150115 Adding image widget support.
** 021 VIG 20150116 Added Menu class support.
*/

package com.goldencode.p2j.ui.client;

import java.io.*;
import java.util.*;
import java.util.logging.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * Common static helpers for the CHUI package.
 * 
 * @author 	SIY
 */
public class UiUtils
{
   /** logger */
   protected static final Logger LOG = LogHelper.getLogger(Image.class.getName());

   /**
    * Check if widget has config.
    * 
    * @param   widget
    *          Widget to check.
    *          
    * @return  <code>true</code> if widget is not null and has config.
    */
   public static boolean hasConfig(Widget<?> widget) {
      return (widget != null && widget.config() != null);
   }

   /**
    * Get ID of specified widget.
    * 
    * @param    widget
    *           Component to for which ID is requested.
    * 
    * @return   The widget ID or {@code null} if the widget doesn't have a config.
    */
   public static WidgetId getWidgetId(Widget<?> widget)
   {
      if (hasConfig(widget))
         return widget.config().id;
      
      return null;
   }

   /**
    * Get ID of specified widget.
    * 
    * @param    widget
    *           Component to for which ID is requested.
    * 
    * @return   The widget ID or -1 if the widget doesn't have a config.
    */
   static public int getWidgetIdAsInt(Widget<?> widget)
   {
      WidgetId wid = getWidgetId(widget);
      
      return wid == null ? -1 : wid.getId();
   }

   /**
    * Check if there are other frames in specified area. Frame passed as a
    * parameter is not counted.
    * 
    * @param    frame 
    *           Frame to exclude.
    * @param    r
    *           Rectangle to check.
    * 
    * @return   <code>true</code> if there are other frames in specified area.
    */
   public static boolean containsFrames(Frame<?> frame, Rectangle r)
   {
      ArrayList<Frame<?>> frames = new ArrayList<Frame<?>>();
      
      Window.findFrames(frames, r, true, frame.isRedirected());
        
      frames.remove(frame);
      
      return frames.isEmpty();
   }

   /**
    * Get ID of the widget which has the input focus.
    *
    * @param   widget
    *          Event source.
    * @return  ID of the widget or <code>-1</code>
    */
   public static WidgetId lookupWidgetId(Widget<?> widget)
   {
      WidgetId widgetId = null;
      
      while (widget != null && widgetId == null)
      {
         if (hasConfig(widget))
            widgetId = widget.config().id;
         
         widget = widget.parent();
      }
      
      return widgetId;
   }

   /**
    * Get ID of the widget which has the input focus.
    *
    * @param   widget
    *          Event source.
    * @return  ID of the widget or <code>-1</code> if the widget doesn't have a config.
    */
   public static int lookupWidgetIdAsInt(Widget<?> widget)
   {
      WidgetId widgetId = lookupWidgetId(widget);
      
      return widgetId == null ? -1 : widgetId.getId();
   }

   /**
    * Get the component which is in focus.
    * 
    * @return   <code>Component</code> instance which is in focus or 
    *           <code>null</code> if there is no focused widget.
    */
   public static Widget<?> getCurrentFocus()
   {
      TitledWindow<?> w = ThinClient.getInstance().isChui() ? Window.resolveWindow() 
                                                            : WindowManager.topWindow();
      return w.currentFocus();
   }

   /**
    * Checks the given event list for global events.
    *
    * @param  list
    *         event list to check 
    * @return <code>true</code> if the list contains at least one global event
    */
   public static boolean checkEventList(EventList list)
   {
      return list.anyEventForWidget(WidgetId.DEFAULT_WINDOW_ID);
   }

   /**
    * Return the frame which contains the specified widget.
    * 
    * @param    widget
    *           Widget for which to find the frame.
    *
    * @return   The frame which contains specified widget or <code>null</code>
    *           if <code>null</code> is passed as the parameter.
    */
   public static Widget<?> locateFrame(Widget<?> widget)
   {
      while (widget != null && !(widget instanceof Frame))
      {
         widget = widget.parent();
         if (widget instanceof DropDown)
         {
            DropDown<?> dd = (DropDown<?>) widget;
            widget = dd.getComboBox();
         }
      }
      
      return widget;
   }

   /**
    * Return the alert box which contains the specified widget.
    *
    * @param    widget
    *           Widget for which to find the alert box.
    *
    * @return   The alert box which contains the specified widget or
    *           <code>null</code> if the widget is not contained in an
    *           alert box or <code>null</code> is passed as the parameter.
    */
   public static AlertBox<?> locateAlertBox(Widget<?> widget)
   {
      while (widget != null && !(widget instanceof AlertBox))
      {
         widget = widget.parent();
      }

      return (AlertBox<?>) widget;
   }

   /**
    * Returns <code>true</code> if the whole widget is not even partially
    * obscured by any frame from the specified array.
    *
    * @param   comp
    *          widget to test
    *
    * @param   above
    *          array of visible frames to test against; normally obtained by
    *          Screen.getFramesAbove() helper
    * @return  <code>true</code> if it is not obscured
    */ 
   public static boolean isAllShowing(Widget<?> comp, Frame<?>[] above)
   {
      if (above.length == 0) 
      {
         return true;
      }
      
      // get the widget's rectangle expressed in screen coordinates
      // some service widgets like BrowseColumn do not have parents and do not
      // take screen space and have to be excluded from the check
      if (comp.parent() == null)
      {
         return false;
      }
   
      Point origin = comp.screenLocation();
      Rectangle bounds = new Rectangle(origin, comp.dimension());
   
      for (int i = 0; i < above.length; i ++) 
      {
         // frame rectangle is (practically) in screen coordinates
         Rectangle rect = above[i].bounds();
         if (rect.intersection(bounds) != null)
         {
            return false;
         }
      }
      return true; 
   }

   /**
    * Check if widget is focusable.
    * 
    * @param   comp
    *          Widget to check.
    *          
    * @return  <code>true</code> if widget us focusable.
    */
   public static boolean isFocusable(Widget<?> comp)
   {
      return hasConfig(comp) && comp.isFocusable();
   }

   /**
    * Hide all dialog frames, if not in the specified list. 
    * 
    * @param   wnd
    *          Containing window.
    * @param   except
    *          The list of dialogs which should not be hidden.
    */
   @SuppressWarnings("rawtypes")
   public static void hideDialogs(Window wnd, Collection<Frame> except)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(wnd);
      
      for (int i = 0; i < c.length; i++)
      {
         if (c[i] instanceof Frame     && 
             ((Frame<?>) c[i]).isDialog() &&
             except.contains(c[i]) == false)
         {
            c[i].setVisible(false);
         }
      }
   }

   /**
    * Get IDs of all frames in the current window.
    * 
    * @param   wnd
    *          Window containing frames, current window.
    * 
    * @return  Array of all frame IDs in their Z-order.
    */
   public static int[] getFrameIDs(Window wnd)
   {
      Widget<?>[] list = UiUtils.getTopLevelComponents(wnd);
      int[] res = new int[list.length];
      int count = 0;
   
      // save IDs of all frames
      for (int i = 0; i < list.length; i++)
      {
         if (!(list[i] instanceof Frame))
            continue;
         
         int wid = getWidgetIdAsInt(list[i]);
         
         res[count ++] = wid;
      }
   
      // produce a compact array
      int[] ids = new int[count];
      for (int i = 0; i < count; i++)
      {
         ids[i] = res[i];
      }
      
      return ids;
   }

   /**
    * Instantiate component using type information provided in configuration.
    * <p>
    * WARNING: at this time, <code>Window</code> and <code>AlertBox</code>
    * cannot be reconstructed via this mechanism.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.         
    * 
    * @return  Instantiated component or <code>null</code> in case of
    *          failure.
    */
   @SuppressWarnings("unchecked")
   public static Widget<?> reconstructComponent(WidgetId id, WidgetConfig cfg)
   {
      //TODO<SIY>: case with missing widget type is not handled here
      // instead we let factory to fail. Probably this is not convenient
      // because some important information (requested type) is lost and can't 
      //be logged nor passed to thrown exception.
      
      Class<?> cfgCls = cfg.getClass();
      
      Class<?> clazz = null;
      if (cfgCls == SkipConfig.class)
      {
         clazz = ((SkipConfig) cfg).vertical ? Skip.class : Space.class;
      }
      else if (cfgCls == FrameConfig.class)
      {
         clazz = Frame.class;
      }
      else if (cfgCls == MenuConfig.class)
      {
         clazz = Menu.class;
      }
      else if (cfgCls == MenuItemConfig.class)
      {
         clazz = MenuItem.class;
      }
      else if (cfgCls == ButtonConfig.class)
      {
         clazz = Button.class;
      }
      else if (cfgCls == RectangleConfig.class)
      {
         clazz = Rect.class;
      }
      else if (cfgCls == SelectionListConfig.class)
      {
         clazz = SelectionList.class;
      }
      else if (cfgCls == ComboBoxConfig.class)
      {
         clazz = ComboBox.class;
      }
      else if (cfgCls == RadioSetConfig.class)
      {
         clazz = RadioSet.class;
      }
      else if (cfgCls == FillInConfig.class)
      {
         clazz = FillIn.class;
      }
      else if (cfgCls == TextConfig.class)
      {
         clazz = Text.class;
      }
      else if (cfgCls == EditorConfig.class)
      {
         clazz = Editor.class;
      }
      else if (cfgCls == BrowseConfig.class)
      {
         clazz = Browse.class;
      }
      else if (cfgCls == BrowseColumnConfig.class)
      {
         clazz = BrowseColumn.class;
      }
      else if (cfgCls == LabelConfig.class)
      {
         clazz = Label.class;
      }
      else if (cfgCls == ToggleBoxConfig.class)
      {
         clazz = ToggleBox.class;
      }
      else if (cfgCls == WindowConfig.class)
      {
         clazz = Window.class;
      }
      else if (cfgCls == ImageConfig.class)
      {
         clazz = Image.class;
      }
   
      return OutputManager.instance().getFactory().create(clazz, id, cfg);
   }

   /**
    * Gets the first frame from the list of currently displayed frames.
    * 
    * @return  the first frame in the list or <code>null</code>
    */
   public static Frame<?> getFirstVisibleFrame()
   {
      Widget<?>[] items = Window.resolveWindow().getWidgets();
   
      Frame<?> firstVisible = null;
      Frame<?> lastDialog   = null;
      Frame<?> frame        = null;
                                                   
      // scan the list of top level items and find the first visible frame
      // if there is a dialog box, find and return the one on the top
      for (int i = 0; i < items.length; i++)
      {
         if (!(items[i] instanceof Frame))
         {
            continue;
         }
   
         frame = (Frame<?>) items[i];
   
         if (frame.isVisible())
         {
            if (frame.isDialog())
            {
               lastDialog = frame;
            }
            else
            {
               if (firstVisible == null)
               {
                  firstVisible = frame;
               }
            }
         }
      }
   
      // return the last dialog if found or else the first visible frame
      if (lastDialog != null)
      {
         return lastDialog;
      }
   
      return firstVisible;
   }

   /**
    * Gets the frame from the list of currently displayed frames that
    * immediately follows the specified frame.
    * 
    * @param   frame
    *          frame that designates the starting point 
    * @return  the next frame in the list or <code>null</code>
    */
   public static Frame<?> getNextVisibleFrame(Frame<?> frame)
   {
      Widget<?>[] items = Window.resolveWindow().getWidgets();
   
      int nref = 0;
                                             
      // scan the list of top level items for the reference point
      for (nref = 0; nref < items.length; nref++)
      {
         if (!(items[nref] instanceof Frame))
         {
            continue;
         }
   
         if (items[nref].equals(frame))
         {
            break;
         }
      }
   
      if (nref == items.length)
      {
         // reference point not found
         return null;
      }
   
      // scan the list of top level items and find the closest visible frame
      for (int i = nref + 1; i < items.length; i++)
      {
         if (!(items[i] instanceof Frame))
         {
            continue;
         }
   
         if (items[i].isVisible())
         {
            return (Frame<?>)items[i];
         }
      }
   
      // no visible frames
      return null;
   }

   /**
    * Returns the frame that is on top of the Z-order or <code>null</code>
    * if no such frame present. If requested method also can filter out all
    * invisible, disabled and out of scope frames.
    * 
    * @param   ignoreState
    *          if <code>true</code> then ignore frame state during search.
    * 
    * @return  The topmost active frame or <code>null</code>.
    */
   public static Frame<?> getTopFrame(boolean ignoreState)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(Window.getDefaultWindow());
   
      // the components list is ordered by Z-order such that the first entry
      // is at the bottom and the last entry is at the top, so search "down"
      // by walking backwards through the array
      for (int i = c.length - 1; i >= 0; i--)
      {
         if (c[i] instanceof Frame)
         {
            Frame<?> frame = (Frame<?>) c[i];
            
            if (ignoreState)
               return frame;
   
            // avoid all cases where a frame is not in scope or is otherwise
            // not a valid frame to be used
            if (!frame.isVisible() ||
                !frame.isEnabled() ||
                !frame.isInScope())
            {
               continue;
            }
         
            // we are done
            return frame;
         }
      }
      
      return null;
   }

   /**
    * Get top visible frame.
    * 
    * @return   reference to top visible frame or <code>null</code> if there is
    *           no such frame.
    */
   public static Frame<?> getTopVisibleFrame()
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(Window.getDefaultWindow());
   
      // the components list is ordered by Z-order such that the first entry
      // is at the bottom and the last entry is at the top, so search "down"
      // by walking backwards through the array
      for (int i = c.length - 1; i >= 0; i--)
      {
         if (c[i] instanceof Frame)
         {
            Frame<?> frame = (Frame<?>) c[i];
   
            if (!frame.isVisible())
               continue;
         
            // we are done
            return frame;
         }
      }
      
      return null;
   }

   /**
    * Get all frames which are visible, overlap given frame and are above it
    * in Z-order.
    * 
    * @param   frame
    *          Reference frame.
    * 
    * @return  Array of found fames.
    */
   public static Frame<?>[] getFramesAbove(Frame<?> frame)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(frame.window());
      ArrayList<Frame<?>> list = new ArrayList<Frame<?>>();
      Rectangle rect = frame.bounds();
   
      // the components list is ordered by Z-order such that the first entry
      // is at the bottom and the last entry is at the top, so search "down"
      // by walking backwards through the array
      for (int i = c.length - 1; i >= 0; i--)
      {
         if (c[i] instanceof Frame)
         {
            Frame<?> frm = (Frame<?>) c[i];
            
            if (frame == frm)
               break;
            
            if (!frm.isVisible() || frame.isRedirected())
               continue;
            
            if (rect.intersects(frm.bounds()))
               list.add(frm);
         }
      }
      
      return list.toArray(new Frame[list.size()]);
   }

   /**
    * Get the first enabled, visible frame, in z-order, after the specified
    * frame. The found frame must have at least one enabled, visible widget.
    * <p>
    * If no frame is specified, it will get the bottommost enabled and visible
    * frame.
    * 
    * @param   frame
    *          Reference frame. When <code>null</null>, it will return the
    *          bottommost enabled and visible frame.
    * 
    * @return  Next enabled frame, above this frame, in z-order, or null if no
    *          such frame exists.
    */
   public static Frame<?> getNextEnabledFrame(Frame<?> frame)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(Window.getDefaultWindow());
      Frame<?> found = null;
   
      // the components list is ordered by Z-order such that the first entry
      // is at the bottom and the last entry is at the top, so search "down"
      // by walking backwards through the array
      for (int i = c.length - 1; i >= 0; i--)
      {
         if (c[i] instanceof Frame)
         {
            Frame<?> frm = (Frame<?>) c[i];
            
            if (frame == frm)
               break;
            
            if (frm.isEnabled() && frm.isVisible() && frm.isAlive())
            {
               int wid = Frame.getFirstFocusableId(frm);
               if (wid != -1)
               {
                  found = frm;
               }
            }
         }
      }
      
      return found;
   }

   /**
    * Get the first enabled, visible frame, in z-order, before the specified
    * frame. The found frame must have at least one enabled, visible widget.
    * <p>
    * If no frame is specified, it will get the bottommost enabled and visible
    * frame.
    * 
    * @param   frame
    *          Reference frame. When <code>null</null>, it will return the
    *          topmost enabled and visible frame.
    * 
    * @return  Previous enabled frame, below this frame, in z-order, or null if
    *          no such frame exists.
    */
   public static Frame<?> getPrevEnabledFrame(Frame<?> frame)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(Window.getDefaultWindow());
      Frame<?> found = null;
      
      // the components list is ordered by Z-order such that the first entry
      // is at the bottom and the last entry is at the top, so search "up"
      // by walking foreword through the array
      for (int i = 0; i < c.length - 1; i++)
      {
         if (c[i] instanceof Frame)
         {
            Frame<?> frm = (Frame<?>) c[i];
            
            if (frame == frm)
               break;
            
            if (frm.isEnabled() && frm.isVisible() && frm.isAlive())
            {
               int wid = Frame.getFirstFocusableId(frm);
               if (wid != -1)
               {
                  found = frm;
               }
            }
         }
      }
      
      return found;
   }

   /**
    * Reset <code>needPause</code> flag for all frames which are visible.
    * 
    * @param   wnd
    *          target window pause.
    */
   public static void resetNeedPause(Window wnd)
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(wnd);
   
      for (Widget<?> component : c)
      {
         if (component.isVisible() && component instanceof Frame)
         {
            Frame<?> frm = (Frame<?>) component;
            
            frm.setNeedPause(false);
         }
      }
   }

   /**
    * Check if any visible frame has <code>needPause</code> flag set.
    * 
    * @return  <code>true</code> if any of the frames on the screen has
    *          <code>needPause</code> flag set to <code>true</code>.
    */
   public static boolean isNeedPause()
   {
      Widget<?>[] c = UiUtils.getTopLevelComponents(Window.getDefaultWindow());
   
      for (Widget<?> component : c)
      {
         if (component.isVisible() && component instanceof Frame)
         {
            Frame<?> frm = (Frame<?>) component;
            
            if (frm.isNeedPause())
               return true;
         }
      }
      
      return false;
   }

   /**
    * Hide frames which are marked as such that can be hidden.
    * <p>
    * Note that if there is an old instance of given frame (not the same
    * frame, but frame with the same name), then method returns false as if
    * there were no frames hidden at all. Logically this is correct because
    * new instance will replace old one at the screen.
    * 
    * @param    current
    *           The frame for which frame hiding is performed.
    * @param    streamed
    *           <code>true</code> hide streamed frames, <code>false</code> -
    *           hide frames from interactive terminal.
    * 
    * @return   Array of boolean results. <br>
    *           First element of array is <code>true</code> if at least one
    *           frame other than given one was hidden and there were no old
    *           instances of given frame.<br>
    *           Second element of array is <code>true</code> if one of hidden
    *           frames has the same row as current frame.
    */
   public static boolean[] hideFrames(Frame<?> current, boolean streamed)
   {
      boolean[] res = new boolean[] {false, false};

      Window wnd = current.window();
      if (wnd == null)
      {
         return res;
      }
      
      Widget<?>[] c = UiUtils.getTopLevelComponents(wnd);
      boolean prevInstance = false;
   
      ArrayList<Frame<?>> list = new ArrayList<Frame<?>>();
      boolean needPause = false;
      
   
      for (Widget<?> component : c)
      {
         if (!component.isVisible() || !(component instanceof Frame))
            continue;
         
         Frame<?> frame = (Frame<?>) component;
         
         if (frame.isCanHide() && 
             !frame.isNoHide() && 
             streamed == frame.isStreamed())
         {
            if (frame != current)
            {
               if (frame.isNeedPause())
                  needPause = true;
   
               list.add(frame);
               
               // In some cases it is possible that frame is instantiated more
               // than once. This happens, for example, with frames which have 
               // NO-HIDE option and which are scoped to procedure which is
               // invoked more than once. There is only one way to check that
               // different Frame instances logically represent the same 4GL 
               // frame - compare names. Fortunately frame names are unique.
               if (frame.name() != null && 
                   current.name() != null &&
                   frame.name().equals(current.name()))
               {
                  prevInstance = true;
               }
            }
         }
      }
   
      try
      {
         if (needPause)
            ThinClient.getInstance().pauseBeforeHide();
      }
      
      finally
      {
         double currentY = (current != null) ? current.screenLocation().y : -1;
         
         for (Frame<?> frame : list)
         {
            if (frame.screenLocation().y == currentY)
               res[1] = true;
               
            frame.setVisible(false);
         }
      }
      
      res[0] = !list.isEmpty() && !prevInstance; 
      
      return res;
   }

   /**
    * Convenience method to retrieve top level frames and other components
    * from the default window.
    * 
    * @return  The array of top level components.
    */
   public static Widget<?>[] getTopLevelComponents(Window wnd)
   {
      return wnd.getContentPane().widgets();
   }

   /**
    * Get DCOLOR attribute for specified widget taking into account color
    * inheritance from parent to child widgets.
    *
    * @param   widget
    *          Widget.
    *
    * @return  found color.
    */
   public static Color getDColor(Widget<?> widget) {
      return getDColor(widget.<BaseConfig>config().dcolor, widget);
   }

   /**
    * Get DCOLOR attribute for specified widget taking into account color
    * inheritance from parent to child widgets. Color inheritance is used only
    * if provided color is <code>null</code>.
    *
    * @param   dcolor
    *          Initial color.
    * @param   widget
    *          Widget.
    *
    * @return  found color.
    */
   public static Color getDColor(Color dcolor, Widget<?> widget)
   {
      Color color = dcolor;

      if (color == null)
      {
         Widget<?> parent = locateFrame(widget);

         if (parent == null || !(parent instanceof Frame) || parent.equals(widget))
            return Color.NORMAL;

         return getDColor(parent.<BaseConfig>config().dcolor, parent);
      }

      return color;
   }

   /**
    * Gets the data array from input stream.
    * 
    * @param    is
    *           The input stream to be read.
    *
    * @return   The image bytes, or {@code null} in case of error.
    */
   public static byte[] getData(InputStream is)
   {
      byte[] imgData = null;

      try
      {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         byte[] b = new byte[512];
         while (is.available() > 0)
         {
            int l = is.read(b);
            baos.write(b, 0, l);
         }
         
         imgData = baos.toByteArray();
      }
      catch (IOException e)
      {
         if (LOG.isLoggable(Level.SEVERE))
         {
            LOG.log(Level.SEVERE, "Could not load image from input stream...", e);
         }
         
         imgData = null;
      }
      finally
      {
         try
         {
            is.close();
         }
         catch (IOException e)
         {
            // ignore
         }
      }
         
      return imgData;
   }
}
